#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include <stdio.h>
#include <util/delay.h>
#include <math.h>
#include <avr/pgmspace.h>

#include "lcd.h"
#include "ds18x20lib.h"

//globalna premenna adc  
volatile unsigned int adc;
volatile unsigned char smer=0;

  
#define CONST_VOLTAGE 13.6e-3 

#define UP 1
#define DOWN 2
  
// prerusenie na akciu skoncenia prevodu
ISR(ADC_vect){  
      adc=ADC; // zapis vysledok prevodu   
}  

// tlacidlo UP 
ISR(INT0_vect){    
   smer = UP;   

} 

   
// tlacidlo DOWN    
ISR(INT1_vect){    
     smer = DOWN;

}   
  

int main(){

	char text[16];

	lcd_init(LCD_DISP_ON);

	lcd_gotoxy(0,0);
	

	ADMUX |=  (1 << REFS0); //referencia 5V  
  	
	// zapnutie AD prevodnika, Free runing, preddelicka 128 pri frek. hodin 8MHz  
    ADCSRA |= (1 << ADEN) | (1 << ADSC) | (1 << ADFR) | (1 << ADIE) | (1 <<ADPS2) | (1 <<ADPS1)  | (1 <<ADPS0); 
	
  	// REZIM 5 - Fast PWM 8bit, bezi na 31,4kHz  
    TCCR1A |= (1 << COM1A1) |  (1 << COM1B1) | (1 << WGM10);  
    TCCR1B |= (1 << WGM12) |  (1 << CS10); // bez preddelicky 
      

    // PD2,PD3 ako vstupn (ext. preruenia)     
    DDRD &= ~((1 << PD2) | (1 << PD3));     
    
    // Zapnutie internho pull-up rezistora    
    PORTD |= (1 << PD2) | (1 << PD3);     
  
    MCUCR |= (1 << ISC01); // doben hrana INT0    
    MCUCR |= (1 << ISC11); // doben hrana INT1    
    GICR |= (1 << INT1) | (1 << INT0); // povol preruenia od INT1 a INT0    
  

  	DDRB |= (1 << PB1) | (1 << PB2) | (1 << PB3);    //Vystup peltier + 2 ventilatory
	PORTB |=  (1 << PB3);

	PORTC |= (1 << PC3); //tlacitko enter
	DDRC &= ~(1 << PC3);	// tlacitko enter

  	OCR1A = 0;   //peltier vypnuty
	OCR1B = 255; // otacky ventilatora naplno
    sei();  //povol globalne prerusenia  


	volatile float cooling_temp=7.0, shutdown_voltage=10.5;
	volatile float temp_in,temp_out;
	char obrazovka=0,thermal_control=0,low_bat=0, high_temp=0;

//  citanie s eeprom, akosi to nefungovalo podla prestav :(
//	cli();
//	if(!(isnan(eeprom_read_float((const float *)1)))) cooling_temp =  eeprom_read_float((const float *)1);
//	if(!(isnan(eeprom_read_float((const float *)5)))) shutdown_voltage = eeprom_read_float((const float *)5);
//	sei();	

	ds1820_init(DS1820_IN);              //Initialize DS1820 Buffer oben
	ds1820_init(DS1820_OUT);              //Initialize DS1820 Buffer mitte
	

	// zaciatok nekonecneho cyklu
	while(1){
	
		lcd_gotoxy(0,0);
		
		temp_in = ds1820_read_temp(DS1820_IN);//zisti vnutornu teplotu
		temp_out = ds1820_read_temp(DS1820_OUT);//zisti teplotu chladica
		
		//  jednotlive obrazovky 
		switch(obrazovka){
		
			case 0:
				sprintf(text,"SET TEMP\n:%.1fC    ",cooling_temp);

				if(smer == UP) {
					smer=0; 
					cooling_temp+=0.5; 
					cli();
					 eeprom_update_float((float *)1, cooling_temp);
					sei();
				}

				if(smer == DOWN) {
					smer=0;
					cooling_temp-=0.5;
					cli();
					eeprom_update_float((float *)1, cooling_temp);
					sei();
				}

				lcd_puts(text);
			break;
			
			case 1:
				sprintf(text,"SET VOLT\n.:%.1fV    ",shutdown_voltage);

				if(smer == UP) {
					smer=0; 
					shutdown_voltage+=0.1;
					cli();
					eeprom_update_float((float *)5, shutdown_voltage);
					sei();
				}

				if(smer == DOWN) {
					smer=0; 
					shutdown_voltage-=0.1;
					cli();
				 	eeprom_update_float((float *)5, shutdown_voltage);
					sei();
				}

				lcd_puts(text);
			break;

			case 2: 
				sprintf(text,"IN TEMP:\n%.1fC    ",temp_in);
				lcd_puts(text);
			break;	

			case 3:
				sprintf(text,"OUT TEMP\n:%.1fC    ",temp_out);
				lcd_puts(text);
			break;
				
			case 4:
				sprintf(text,"VOLTAGE:\n%.1fV     ",(float)adc*CONST_VOLTAGE);
				lcd_puts(text);
			break;	

			case 5:
				sprintf(text,"REG PELT\nIER:%d    ",OCR1A);
				lcd_puts(text);
			break;	

			case 6:
				sprintf(text,"REG VENT\n.:%d    ",OCR1B);
				lcd_puts(text);
			break;	


			case 7:
				sprintf(text,"Thermal:\n%d    ",thermal_control);
				lcd_puts(text);
			break;	

			case 8:
				if(low_bat) sprintf(text,"LOW BATT\nERY!!!  ");
				else sprintf(text,"BATTERY \nOK!        ");
				lcd_puts(text);
			break;	
		
		
		}

		if((PINC & (1 << PC3)) == 0) {obrazovka++; _delay_ms(100);}
		if(obrazovka == 9) {obrazovka = 0;}

		// ak je stav baterie v poriadku a nieje kriticka teplota
		if((low_bat == 0) && (high_temp == 0)){

			// ak je vnutorna teplota vyssia ako nastavena
			if(temp_in > cooling_temp){

				OCR1B = 255; //maximalne chladenie	
			
	
				//riadenie teploty peltieru kvoli jeho ucinnosti
				//ak je rozdiel viac ako 35C tak....
				if((temp_out - temp_in) > 35.0){
					thermal_control = 1;
					TCCR1A |= (1 << COM1A1);
					// zacni znizovat vykon peltieru
					if((temp_out - temp_in) < 45 ) { OCR1A = ((temp_out - temp_in) - 35.0) * 25; }
					else { OCR1A = 0;};
				
				}else{
					// inak pusti plny vykon :)
					thermal_control = 0;
					TCCR1A &= ~(1 << COM1A1);
					PORTB &= ~(1 << PB1);
					OCR1A=0;
				}


			}else{
				// ak je teplota nizsia ako nastavena
				TCCR1A |= (1 << COM1A1);

				thermal_control = 0;
		
				// ak je rozdiel teplot od 25-35C tak reguluj ventilator
				if((temp_out - temp_in) > 25.0 &&  (temp_out - temp_in) < 35.0) { OCR1B = ((temp_out - temp_in) - 25.0) * 25; }
				else if((temp_out - temp_in) < 25.0) {OCR1B = 0;}	// ak je mensi ako 25
				else {OCR1B=255;}

				// reguluj vykon peltiera pre udrzanie teploty
				if((cooling_temp - temp_in) < 1){	
						OCR1A = (cooling_temp - temp_in)*0xFF;
				}else{
					OCR1A = 245;
				}
		
		
			}
		}

		//thermal control//
		
		// kriticka teplota
		if((temp_out > 50.0) && (high_temp == 0)){
			high_temp=1;
			obrazovka = 3;
			TCCR1A |= (1 << COM1A1);
			OCR1A = 255; // peltier vypnuty
			OCR1B = 255; //maximalne chladenie
		
		//hysterezia
		}else if(temp_out < 45.0){
			high_temp=0;
		}


		//BATTERY CONTROL//
		
		// kriticky nizky stav baterie
		if(((float)adc*CONST_VOLTAGE < shutdown_voltage) & (low_bat == 0)) {
			low_bat = 1;
			obrazovka = 8;
						
			OCR1A = 255; // peltier vypnuty
			OCR1B = 0;		//ventor vonkajsi vypnuuty
			PORTB &=  ~(1 << PB3); // vnutorny ventilator vypnuty
					
		// hysterezia
		}else if((float)adc*CONST_VOLTAGE > shutdown_voltage+0.5){
			low_bat=0;
			PORTB |=  (1 << PB3);
		}

	




	}
 
}
